import express from "express"
import mariadb from "mariadb"
import cors from "cors"

import devices from "./routes/devices.js"
import points from "./routes/points.js"

const app = express();
const port = process.env.PORT || 80; // načte senastavení z Dockeru / defaultní hodnota
const db_host = process.env.DB_HOST || "localhost";
const db_port = process.env.DB_PORT || 3306;
const db_username = process.env.USERNAME || "root";
const db_password = process.env.DB_PASSWORD || "admin";

const db = await mariadb.createConnection({
    host: db_host,
    port: db_port,
    user: db_username,
    password: db_password,
    database: 'flowers'
});

//keep db connection alive with pings, else the DB will hang up the connection
setInterval(() => { db.ping() }, 1000 * 60 * 10); //ping aby baze nepadla

app.set("db", db);

app.use(express.json({ limit: "10mb" }));
app.use(express.static("static"));
app.use(cors());

app.use("/api", devices);
app.use("/api", points);

app.listen(port, () => {
    console.log(`App listening on port ${port}`)
});