import express from "express"
import dateToSQL from "../lib/dateToSQL.js";
import ErrorAgregator from "../lib/errorAgregator.js";

const router = express.Router();

router.get("/points/:deviceId", async (req, res) => {
    const db = req.app.get("db");

    const deviceId = Number.parseInt(req.params.deviceId);

    if (isNaN(deviceId)) {
        res.send({
            error: "Id must be a number"
        })
        return;
    }

    const deviceResult = await db.query(` 
        SELECT 
        devices.id

        FROM 
            devices 

        WHERE
            devices.id = ${deviceId}


        GROUP BY
            devices.id
    `);

    if (deviceResult === null || deviceResult === undefined || deviceResult.length === 0) {
        res.status(404);
        res.send({
            error: "Device not found"
        })
        return;
    }

    const devicesResult = await db.query(`
        SELECT 
            devices.id,
            devices.name, 
            data_points.point_transmited_datetime as "point_datetime",
            data_points.value as "point_value"   

        FROM 
            devices 
            
        LEFT JOIN 
            data_points  

        ON
            data_points.device_id = devices.id

        WHERE
            devices.id = ${deviceId}

        ORDER BY
            data_points.point_transmited_datetime ASC
    `);

    if (devicesResult.length === 0) {
        res.send();
        return;
    }

    const resObject = {
        id: devicesResult[0].id,
        data_points: devicesResult.map((deviceResult) => {
            if (deviceResult.point_datetime !== null &&
                deviceResult.point_datetime !== undefined &&
                deviceResult.point_value !== null &&
                deviceResult.point_value !== undefined
            ) {
                return {
                    point_datetime: deviceResult.point_datetime,
                    point_value: deviceResult.point_value
                }
            }
        }).filter(element => element)
    };

    res.send(resObject);
});


/*
    REQ
    {
        point_value: number,
        point_datetime: string Date
    }
*/
router.post("/points/:id", async (req, res) => {
    const db = req.app.get("db");

    const deviceId = Number.parseInt(req.params.id);

    if (isNaN(deviceId)) {
        res.status(400);
        res.send({
            error: "Id must be a number"
        })
        return;
    }

    const errorAgregator = new ErrorAgregator();

    if (req.body.point_value == null || Number.isNaN(Number.parseInt(req.body.point_value))) {
        errorAgregator.pushError("point_value", "point_value must be a number");
    }

    if (req.body.point_datetime == null || Number.isNaN(Date.parse(req.body.point_datetime))) {
        errorAgregator.pushError("point_datetime", "point_datetime must be in format YYYY-MM-DDTHH:mm:ss.sssZ");
    }

    if (errorAgregator.errors.length > 0) {
        res.status(400);
        res.send(errorAgregator);
        return;
    }

    const receivedDate = new Date();

    const insertResult = await db.query(`
        INSERT INTO
            data_points (device_id, value, point_measured_datetime, point_transmited_datetime)

        VALUES
        (${deviceId}, ${req.body.point_value}, "${dateToSQL(new Date(Date.parse(req.body.point_datetime)))}", "${dateToSQL(receivedDate)}")
    `);


    const selectResult = await db.query(`
        SELECT
            data_points.device_id,
            data_points.value as "point_value",
            data_points.point_measured_datetime as "point_datetime"

        FROM
            data_points

        WHERE
            data_points.id = ${Number.parseInt(insertResult.insertId)}

    `);

    res.status(201);
    res.send(selectResult);
});

export default router;