import { useLocalSearchParams, router } from 'expo-router';
import { SafeAreaProvider, SafeAreaView } from 'react-native-safe-area-context';
import { View, StyleSheet, Image, Platform, ScrollView } from 'react-native';
import { Appbar, Text, ActivityIndicator, Icon } from 'react-native-paper';
import { useEffect, useRef, useState } from "react"

import axios from "axios"

import PointsHistogram from '../../components/PointsHistogram';


export default function DeviceDetail() {
    const { id } = useLocalSearchParams();

    const graphContainer = useRef();

    const [device, setDevice] = useState();
    const [devicePoints, setDevicePoints] = useState();
    const [width, setWidth] = useState(0);
    const [height, setHeight] = useState(0);

    function extractViewSize(onLayoutEvent) {
        setWidth(onLayoutEvent.nativeEvent.layout.width);
        setHeight(onLayoutEvent.nativeEvent.layout.height);
    }

    useEffect(() => {
        async function getDeviceById(id) {
            const result = await axios.get(`http://${global.API_URL}/api/devices/${id}`);
            await new Promise((resolve) => { setTimeout(resolve, 1000) });
            setDevice(result.data);
        }

        async function getDevicePointsById(id) {
            const result = await axios.get(`http://${global.API_URL}/api/points/${id}`);
            await new Promise((resolve) => { setTimeout(resolve, 1000) });
            setDevicePoints(result.data);
        }

        getDeviceById(id);
        getDevicePointsById(id);
    }, [])

    return (
        <SafeAreaProvider>
            <SafeAreaView style={styles.container}>
                <Appbar.Header style={styles.header}>
                    <Appbar.BackAction onPress={() => { router.navigate("/") }}></Appbar.BackAction>
                    <Appbar.Content title="Device detail" />
                    <Appbar.Action icon="pencil" onPress={() => { router.push(`/settings/${id}`) }} />
                </Appbar.Header>
                <View style={styles.contentContainer}>
                    {
                        device ? (
                            <View
                                style={{
                                    width: "100%",
                                    justifyContent: "flex-start",
                                    marginTop: 20,
                                    marginBottom: 20,
                                    maxWidth: 1000,
                                    flex: 1,
                                    alignSelf: "center",
                                    alignItems: "center"
                                }}
                            >
                                <View style={{
                                    width: "100%",
                                    height: 250,
                                    borderRadius: 1000,
                                    borderWidth: 4,
                                    borderColor: "rgba(234, 221, 255, 1)",
                                    justifyContent: "center",
                                    alignItems: "center"
                                }}>
                                    {
                                        device.pic_url ? (
                                            <Image
                                                style={{
                                                    width: "100%",
                                                    height: "100%",
                                                    borderRadius: 1000
                                                }}
                                                source={{
                                                    uri: `http://${global.API_URL}${device.pic_url}`
                                                }}
                                            ></Image>
                                        ) :
                                            (
                                                <Icon source={"image-broken-variant"} size={70} color="rgba(56, 30, 114, 1)"></Icon>
                                            )
                                    }
                                </View>
                                <Text variant='titleLarge' style={{ marginTop: 20 }}>{device.name}</Text>
                                {
                                    devicePoints !== null && devicePoints !== undefined ? (
                                        devicePoints.data_points && devicePoints.data_points.length > 0 ? (
                                            <ScrollView onLayout={extractViewSize} style={{ flex: 1, width: "100%", minHeight: 350 }} ref={graphContainer} scrollEnabled={false}>
                                                <PointsHistogram devicePoints={devicePoints} width={width} height={height}></PointsHistogram>
                                            </ScrollView>
                                        ) : (
                                            <View style={{ flex: 1, width: "100%", justifyContent: "center", alignItems: "center" }}>
                                                <Text variant='bodyLarge'>No data</Text>
                                            </View>
                                        )
                                    )
                                        :
                                        <View style={{ flex: 1, justifyContent: "center", alignItems: "center" }}>
                                            <ActivityIndicator size={"large"}></ActivityIndicator>
                                            <Text style={{ marginTop: 10 }} variant='titleLarge'>Loading...</Text>
                                        </View>
                                }
                            </View>
                        ) : (
                            <View style={{ flex: 1, justifyContent: "center", alignItems: "center" }}>
                                <ActivityIndicator size={"large"}></ActivityIndicator>
                                <Text style={{ marginTop: 10 }} variant='titleLarge'>Loading...</Text>
                            </View>
                        )
                    }
                </View>
            </SafeAreaView>
        </SafeAreaProvider >
    );
}

const styles = StyleSheet.create({
    container: {
        flex: 1,
        backgroundColor: 'rgba(234, 221, 255, 1)'
    },
    header: {
        backgroundColor: 'rgba(234, 221, 255, 1)'
    },
    activityIndicatorContainer: {
        flex: 1,
        justifyContent: "center",
        alignContent: "center",
        backgroundColor: 'white'
    },
    contentContainer: {
        flex: 1,
        backgroundColor: "white",
        padding: 10
    },
});