import { useEffect, useState } from "react"
import { StyleSheet, View, Platform } from 'react-native';
import { SafeAreaProvider, SafeAreaView } from 'react-native-safe-area-context';
import { ActivityIndicator, Appbar, Text, Button } from 'react-native-paper';
import axios from "axios"

import DeviceList from '../components/DeviceList';

global.API_URL = "95.216.185.195:80"; //Platform.OS === "web" ? "localhost" : "10.0.2.2";

export default function App() {
  const [loading, setLoading] = useState(true);
  const [devices, setDevices] = useState();

  async function fetchData() {
    setLoading(true);
    try {
      const result = await axios.get(`http://${global.API_URL}/api/devices`, { timeout: 5000 });
      await new Promise((resolve) => {
        setTimeout(resolve, 1000);
      })
      setDevices(result.data);
      setLoading(false);
    }
    catch (exc) {
      console.log(JSON.stringify(exc, null, 2));
    }
  }

  useEffect(() => {
    fetchData();
  }, []);


  return (
    <SafeAreaProvider>
      <SafeAreaView style={styles.container}>
        <Appbar.Header style={styles.header}>
          <Appbar.Content title="Devices" />
        </Appbar.Header>
        {
          loading === false ? (
            Array.isArray(devices) && devices.length > 0 ?
              <View style={styles.devicesContainer}>
                <DeviceList devices={devices} onRefresh={fetchData}></DeviceList>
              </View> :
              <View style={{ flex: 1, justifyContent: "center", alignItems: "center", backgroundColor: "white", gap: 10 }}>
                <Text variant="bodyLarge">No devices found</Text>
                <Button mode="contained" onPress={fetchData}>Retry</Button>
              </View>
          ) : (
            <View style={styles.activityIndicatorContainer}>
              <View style={{ flex: 1, justifyContent: "center", alignItems: "center" }}>
                <ActivityIndicator size={"large"}></ActivityIndicator>
                <Text style={{ marginTop: 10 }} variant='titleLarge'>Loading...</Text>
              </View>
            </View>
          )
        }
      </SafeAreaView>
    </SafeAreaProvider>
  );
}

const styles = StyleSheet.create({
  container: {
    flex: 1,
    backgroundColor: 'rgba(234, 221, 255, 1)'
  },
  header: {
    backgroundColor: 'rgba(234, 221, 255, 1)'
  },
  activityIndicatorContainer: {
    flex: 1,
    justifyContent: "center",
    alignContent: "center",
    backgroundColor: 'white'
  },
  devicesContainer: {
    flex: 1,
    backgroundColor: "white"
  },
});