import { StyleSheet, Image, Pressable, View } from "react-native"
import { Surface, Text, Icon } from 'react-native-paper';
import { Link } from 'expo-router';

import NumberRange from "./NumberRange";

export default function DeviceItem({ device }) {
    return (
        <Link href={`/device/${device.id}`} style={{ flexDirection: "row", justifyContent: "center" }} asChild>
            <Pressable>
                <Surface style={styles.container}>
                    {
                        device.pic_url ? (
                            <Image style={styles.devicePic} source={{ uri: `http://${global.API_URL}${device.pic_url}` }}></Image>
                        ) : (
                            <View style={styles.devicePic}>
                                <Icon source={"image-broken-variant"} size={30} color="rgba(56, 30, 114, 1)"></Icon>
                            </View>
                        )
                    }
                    <View style={styles.deviceNameContainer}>
                        <Text variant="titleMedium">{device.name}</Text>
                        <View style={styles.numberRangeContainer}>
                            <Icon source={"water-outline"} size={20} />
                            <NumberRange styles={styles.numberRange} minThreshold={device.min_threshold} maxThreshold={device.max_threshold} value={device.last_data_point_value}></NumberRange>
                            <Icon source={"water"} size={20} />
                        </View>
                    </View>
                    <Text style={{ display: "flex", alignItems: "center", textAlignVertical: "center" }}>{device.last_data_point_value ? device.last_data_point_value : 0}%</Text>
                </Surface>
            </Pressable>
        </Link>
    )
}

const styles = StyleSheet.create({
    container: {
        flex: 1,
        flexDirection: "row",
        alignItems: "stretch",
        justifyContent: "space-evenly",
        padding: 10,
        borderRadius: 5,
        maxWidth: 800
    },
    devicePic: {
        width: 100,
        height: 100,
        borderColor: "rgba(56, 30, 114, 1)",
        borderRadius: 100,
        borderWidth: 1,
        objectFit: "cover",
        justifyContent: "center",
        alignItems: "center"
    },
    deviceNameContainer: {
        display: "flex",
        flex: 1,
        gap: 5,
        justifyContent: "center",
        alignItems: "center"
    },
    numberRangeContainer: {
        display: "flex",
        flexDirection: "row",
        justifyContent: "center",
        alignItems: "center",
        alignSelf: "stretch"
    },
    numberRange: {
        width: "70%",
        height: 15,
    }
});