import { LinearGradient } from "expo-linear-gradient"
import { useState } from "react"
import { StyleSheet, View } from "react-native"

export default function NumberRange({ minThreshold, maxThreshold, value, styles }) {
    const [width, setWidth] = useState(1);
    const [height, setHeight] = useState(1);

    function extractViewSize(onLayoutEvent) {
        setWidth(onLayoutEvent.nativeEvent.layout.width);
        setHeight(onLayoutEvent.nativeEvent.layout.height);
    }

    function scale(number, inMin, inMax, outMin, outMax) {
        if (number < inMin) number = inMin;
        if (number > inMax) number = inMax;
        return (number - inMin) * (outMax - outMin) / (inMax - inMin) + outMin;

    }

    return (
        <View style={{ ...localStyles.container, ...styles }} onLayout={extractViewSize}>
            <LinearGradient
                style={localStyles.gradient}
                start={{ x: 0.0, y: 0 }}
                end={{ x: 1, y: 0 }}
                locations={[0, scale(minThreshold, 0, 100, 0, 1), scale(maxThreshold, 0, 100, 0, 1), 1]}
                colors={["rgba(232, 65, 65, 1)", "rgba(50, 168, 82, 1)", "rgba(50, 168, 82, 1)", "rgba(232, 65, 65, 1)"]}>
                <View style={{ ...localStyles.minThreshold, height, left: scale(minThreshold, 0, 100, 6, width - 6) }}></View>
                <View style={{ ...localStyles.maxThreshold, height, left: scale(maxThreshold, 0, 100, 6, width - 6) }}></View>
                <View style={{ ...localStyles.value, height, left: scale(value, 0, 100, 6, width - 6) }}></View>
            </LinearGradient>
        </View>
    )
}

const localStyles = StyleSheet.create({
    container: {
        borderRadius: 10,
        flexDirection: "row",
        overflow: "hidden"
    },
    gradient: {
        borderRadius: 10,
        flex: 1,
        overflow: "hidden"
    },
    minThreshold: {
        width: 2,
        backgroundColor: "white",
        position: "absolute",
        left: "25%",
    },
    maxThreshold: {
        width: 2,
        backgroundColor: "white",
        position: "absolute",
        right: "25%",
    },
    value: {
        width: 7,
        backgroundColor: "rgba(56, 30, 114, 1)",
        position: "absolute",
    }
})