import { VictoryChart, VictoryLine, VictoryTheme, VictoryAxis, VictoryCursorContainer, VictoryLabel } from "victory-native"

export default function PointsHistogram({ width, height, devicePoints }) {
    function formatLabelDate(millis) {
        const date = new Date(millis);
        let label = date.getDate() + "." + date.getMonth() + ". ";
        label += String(date.getHours()).length === 1 ? "0" + date.getHours() + ":" : date.getHours() + ":";
        label += String(date.getMinutes()).length === 1 ? "0" + date.getMinutes() : date.getMinutes();

        return label;
    }

    return (
        devicePoints && <VictoryChart
            theme={VictoryTheme.material}
            standalone={true}
            containerComponent={
                <VictoryCursorContainer
                    cursorLabelOffset={{ x: -46, y: -60 }}
                    cursorLabelComponent={<VictoryLabel style={{ fill: "rgba(127, 103, 190, 1)" }} />}
                    cursorLabel={({ datum }) => {
                        let label = "date: " + formatLabelDate(Math.floor(datum.x)) + "\n";
                        label += "moisture: " + Math.floor(datum.y) + "%"
                        return label;
                    }}
                />}
            padding={{ left: 40, top: 25, bottom: 35, right: 25 }}
        >
            <VictoryLine
                style={{
                    data: { stroke: "rgba(79, 55, 139, 1)" },
                    parent: { border: "1px solid #ccc" }
                }}
                interpolation={'natural'}
                padding={0}
                domain={
                    {
                        y: [0, 100]
                    }
                }
                data={devicePoints.data_points.map((point) => {
                    return {
                        x: new Date(point.point_datetime).getTime(),
                        y: point.point_value
                    }
                })}
            />
            <VictoryAxis dependentAxis tickFormat={(tick) => `${tick}%`} />
            <VictoryAxis
                fixLabelOverlap
                tickFormat={(tick) => formatLabelDate(tick)}
            />
        </VictoryChart>
    )
}