#include <ArduinoJson.h>
#include <ESP8266HTTPClient.h>
#include <ESP8266WiFi.h>
#include <WiFiClient.h>

WiFiClient client;
HTTPClient http;

const char* nazevwifi = "nazevwifi";
const char* heslowifi = "heslowifi";
const char* serverAddress = "192.168.0.190";

unsigned long lastResistance = 0;
unsigned long lastCapacitance = 0;
int milisDelayResistance = 1000 * 60;
int milisDelayCapacitance = 1000 * 60;
String lastDataPoint = "";

struct sensorData {
  String resistance;
  String capacitance;
};

String getLastData(String data, char delimiter) {   //extrakce posledni casti retezce, podle oddelovace = delimetru
  int index = 0;                                    // projede se string, pokud se narazi na oddelovac, zmeni se index
                                                    //nakonec se vrati cast stringu o jednu hodnotu za poslednim oddelovacem
  for(int i = 0; i < data.length(); i++) {
    if(data[i] == delimiter) {
      index = i;
    }
  }
  return data.substring(index + 1);
}

struct sensorData {
  String resistance;
  String capacitance;
};

sensorData extractData(String data, char delimiter) {     //extrakce dat z textu, oddelenych delimetrem
  struct sensorData splitData;                            //struktura ma 2 casti - kapacitni a odporovou

  int index = 0;
  
  for(int i = 0; i < data.length(); i++) {
    if(data[i] == delimiter) {
      index = i;
    }
  }

  splitData.resistance = data.substring(0, index);
  splitData.capacitance = data.substring(index + 1);
  

  return splitData;
}

int scale (int number,int inMin,int inMax,int outMin,int outMax) {             //prevadi namerene hodnoty na %
    return (number - inMin) * (outMax - outMin) / (inMax - inMin) + outMin;    //in = puvodni rozsah, out = novy rozsah
}

void sendData(int deviceId, int data, HTTPClient *httpClient) {   //odesilani dat na HTTP endpoint pomoci POST
    int httpResponseCode = 0;
    
    httpClient->begin(client, "http://95.216.185.195:80/api/points/" + String(deviceId)); //zahajeni komunikace se serverem a konkretnim endpointem
    httpClient->addHeader("Content-Type", "application/json");
     
    httpResponseCode = httpClient->POST("{\"point_value\": " + String(data) + ",\"point_datetime\": \"2024-03-14T20:09:00.000Z\"}");
    //odeslani POST pozadavku 
    
    if(httpResponseCode > 0) {      //pokud dojde z zápisu, vypise se odpoved na seriovou linku, jinak se vypise chybova hlaska
      String response = httpClient->getString();
      Serial.println(httpResponseCode);
      Serial.println(response);
    } else {
      Serial.print("HTTP POST failed, error: ");
      Serial.println(httpResponseCode);
    }
    
    httpClient->end();
}

int getDelaySettings(int deviceId, HTTPClient *httpClient) {    //nastaveni prodlevy na odesilani dat endpointům
    int httpResponseCode = 0;
    
    httpClient->begin(client, "http://95.216.185.195:80/api/devices/" + String(deviceId));
     
    httpResponseCode = httpClient->GET();
    
    if(httpResponseCode > 0) {
      String response = httpClient->getString();
      Serial.println(httpResponseCode);
      Serial.println(response);
      
      JsonDocument doc;
      deserializeJson(doc, response);

      return doc["update_frequency_seconds"];
    } else {
      Serial.print("HTTP POST failed, error: ");
      Serial.println(httpResponseCode);
    }
    
    httpClient->end();
    return -1;
}

void setup() {
  Serial.begin(9600); // Nastavení rychlosti sériové komunikace na 9600 baudů

  WiFi.begin(nazevwifi,heslowifi);

  Serial.println("Připojuji se k WiFi...");
  
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.println("Připojeno k Wi-Fi síti");

  milisDelayResistance = getDelaySettings(1, &http) * 1000;     // ziskani nastaveni hodnoty odporoveho endpointu
  milisDelayCapacitance = getDelaySettings(2, &http) * 1000;    //
  lastResistance = - milisDelayResistance;
  lastCapacitance = - milisDelayCapacitance;
}

void loop() {
if (WiFi.status() == WL_CONNECTED) {
    //char body[200] = "{\"point_value\": 55 ,\"point_datetime\": \"2024-03-14T20:09:00.000Z\"}"
     
    String receivedString = Serial.readString();    //prijeti dat z Arduina
    if(receivedString != "") {
       lastDataPoint = getLastData(receivedString, ';'); //ziskani poslednich/nejaktualnejsich dat a rozdeleni pomoci ;
    } 
    struct sensorData parsedData = extractData(lastDataPoint, ','); //rozdeleni prijatych dat pomoci , na kapacitni a odporovou hodnotu
     
    Serial.println("Received: " + lastDataPoint); // Výpis přijatého řetězce
    Serial.println("Resistance: " + parsedData.resistance);
    Serial.println("Capacitance: " + parsedData.capacitance);
  
    int scaledResistance = scale(parsedData.resistance.toInt(), 1050, 200, 0, 100); //prevedeni hodnoty na procenta
    int scaledCapacitance = scale(parsedData.capacitance.toInt(), 850, 400, 0, 100);
    Serial.println("scaled resistance: " + String(scaledResistance) + "%");
    Serial.println("scaled capacitance: " + String(scaledCapacitance) + "%");

    if(millis() - lastResistance >= milisDelayResistance && parsedData.resistance.toInt() != 0 && scaledResistance < 100 && scaledResistance > 0) {
      //poslani dat, pokud data nejsou 0, nebo nedoslo k chybe mereni zpusobene senzorem
      lastResistance = millis();
      sendData(1, scaledResistance, &http);
    }

    if(millis() - lastCapacitance >= milisDelayCapacitance && parsedData.capacitance.toInt() != 0 && scaledCapacitance < 100 && scaledCapacitance > 0) {
      lastCapacitance = millis();
      sendData(2, scaledCapacitance, &http);
    }

    milisDelayResistance = getDelaySettings(1, &http) * 1000;
    milisDelayCapacitance = getDelaySettings(2, &http) * 1000;

    Serial.println(milisDelayResistance);
    Serial.println(milisDelayCapacitance);

    delay (10000);
  } 
} 
